#include "sal/config.h"
#include "cppuhelper/factory.hxx"
#include "cppuhelper/implementationentry.hxx"
#include "cppuhelper/implbase2.hxx"
#include "com/sun/star/lang/XServiceInfo.hpp"
#include "com/sun/star/smarttags/XSmartTagAction.hpp"
#include "com/sun/star/container/XStringKeyMap.hpp"
#include "com/sun/star/lang/XMultiServiceFactory.hpp"
#include "com/sun/star/system/XSystemShellExecute.hpp"
#include "com/sun/star/system/SystemShellExecuteFlags.hpp"

#include <libxml/xmlmemory.h>
#include <libxml/parser.h>
#include <libxml/xpath.h>
#include <libxml/xpathInternals.h>
#include <unotools/bootstrap.hxx>
#include <osl/file.hxx>
#include <vector>

#define C2U(cChar) rtl::OUString::createFromAscii(cChar)

void lcl_GetFileList( rtl::OUString aDirectory, std::vector<rtl::OUString>& rFileList )
{
    // iterate through directory contents
    osl::FileStatus aFileStatus(FileStatusMask_All);
    osl::DirectoryItem aDirItem;
    osl::Directory aDir( aDirectory + C2U("/smarttags"));
    aDir.open();
    while ( osl::FileBase::E_None == aDir.getNextItem( aDirItem ) )
    {
        aDirItem.getFileStatus(aFileStatus);
        if ( osl::FileStatus::Regular == aFileStatus.getFileType() )
        {
            rtl::OUString ustrSystemPath;
            osl::FileBase::getSystemPathFromFileURL( aFileStatus.getFileURL(), ustrSystemPath );
            rFileList.push_back(ustrSystemPath);
        }
    }
    aDir.close();
}


// component helper namespace
namespace comp_MOSTLSmartTagAction {

namespace css = ::com::sun::star;

// component and service helper functions:
::rtl::OUString SAL_CALL _getImplementationName();
css::uno::Sequence< ::rtl::OUString > SAL_CALL _getSupportedServiceNames();
css::uno::Reference< css::uno::XInterface > SAL_CALL _create( css::uno::Reference< css::uno::XComponentContext > const & context );

} // closing component helper namespace


/// anonymous implementation namespace
namespace {

namespace css = ::com::sun::star;

/** Class: MOSTLAction
*/
class MOSTLAction
{
private:
    rtl::OUString maUrl;
    rtl::OUString maCaption;
    sal_Int32     mnActionID;

public:
    MOSTLAction( const rtl::OUString aCaption, const rtl::OUString aUrl, const sal_Int32 nActionID ) :
        maUrl(aUrl), maCaption (aCaption), mnActionID(nActionID) { }

    inline rtl::OUString getCaption() const { return maCaption; }
    inline rtl::OUString getUrl() const { return maUrl; }
    inline sal_Int32     getActionID() const { return mnActionID; }
};

/** Class: MOSTLActionData
*/
class MOSTLActionData
{
private:

    rtl::OUString aCaption;
    rtl::OUString aSmartTagType;
    std::vector<MOSTLAction> aActions;

public:
    inline rtl::OUString getSmartTagType() const { return aSmartTagType; }
    inline void setSmartTagType(rtl::OUString aNewName) { this->aSmartTagType = aNewName; }

    inline rtl::OUString getCaption() const { return aCaption; }
    inline void setCaption( rtl::OUString aNewCaption ) { this->aCaption = aNewCaption; }

    inline sal_Int32 getActionCount() const { return aActions.size(); }
    rtl::OUString    getActionCaption(sal_Int32 nActionIndex) const;
    rtl::OUString    getActionUrl(sal_Int32 nActionIndex) const;
    sal_Int32        getActionID(sal_Int32 nActionIndex) const;

    inline void addAction(MOSTLAction aAction) { aActions.push_back(aAction); }
};


class MOSTLSmartTagAction:
    public ::cppu::WeakImplHelper2<
        css::lang::XServiceInfo,
        css::smarttags::XSmartTagAction>
{
public:
    explicit MOSTLSmartTagAction(css::uno::Reference< css::uno::XComponentContext > const & context);

    // ::com::sun::star::lang::XServiceInfo:
    virtual ::rtl::OUString SAL_CALL getImplementationName() throw (css::uno::RuntimeException);
    virtual ::sal_Bool SAL_CALL supportsService(const ::rtl::OUString & ServiceName) throw (css::uno::RuntimeException);
    virtual css::uno::Sequence< ::rtl::OUString > SAL_CALL getSupportedServiceNames() throw (css::uno::RuntimeException);

    // ::com::sun::star::lang::XInitialization:
    virtual void SAL_CALL initialize(const css::uno::Sequence< ::com::sun::star::uno::Any > & aArguments) throw (css::uno::RuntimeException, css::uno::Exception);

    // ::com::sun::star::smarttags::XSmartTagAction:
    virtual ::sal_Int32 SAL_CALL getSmartTagCount() throw (css::uno::RuntimeException);
    virtual ::rtl::OUString SAL_CALL getName(const css::lang::Locale & aLocale) throw (css::uno::RuntimeException);
    virtual ::rtl::OUString SAL_CALL getDescription(const css::lang::Locale & aLocale) throw (css::uno::RuntimeException);
    virtual ::rtl::OUString SAL_CALL getSmartTagName(::sal_Int32 nSmartTagIndex) throw (css::uno::RuntimeException, css::lang::IndexOutOfBoundsException);
    virtual ::rtl::OUString SAL_CALL getSmartTagCaption(::sal_Int32 nSmartTagIndex, const css::lang::Locale & aLocale) throw (css::uno::RuntimeException, css::lang::IndexOutOfBoundsException);
    virtual ::sal_Int32 SAL_CALL getActionCount(const ::rtl::OUString & aSmartTagName, const css::uno::Reference< css::frame::XController > & xController) throw (css::uno::RuntimeException);
    virtual ::sal_Int32 SAL_CALL getActionID(const ::rtl::OUString & aSmartTagName, ::sal_Int32 nActionIndex, const css::uno::Reference< css::frame::XController > & xController) throw (css::uno::RuntimeException, css::lang::IllegalArgumentException);
    virtual ::rtl::OUString SAL_CALL getActionCaptionFromID(::sal_Int32 nActionID, const ::rtl::OUString & aApplicationName, const css::lang::Locale & aLocale, const css::uno::Reference< css::container::XStringKeyMap > & xProperties, const ::rtl::OUString & aText, const ::rtl::OUString & aXML, const css::uno::Reference< css::frame::XController > & xController, const css::uno::Reference< css::text::XTextRange > & xTarget) throw (css::uno::RuntimeException, css::lang::IllegalArgumentException);
    virtual ::rtl::OUString SAL_CALL getActionNameFromID(::sal_Int32 nActionID, const css::uno::Reference< css::frame::XController > & xController) throw (css::uno::RuntimeException, css::lang::IllegalArgumentException);
    virtual void SAL_CALL invokeAction(::sal_Int32 nActionID, const ::rtl::OUString & aApplicationName, const css::uno::Reference< css::frame::XController > & xController, const css::uno::Reference< css::text::XTextRange > & xTarget, const css::uno::Reference< css::container::XStringKeyMap > & xProperties, const ::rtl::OUString & aText, const ::rtl::OUString & aXML, const css::lang::Locale & aLocale) throw (css::uno::RuntimeException, css::lang::IllegalArgumentException);
    virtual ::sal_Bool SAL_CALL isCaptionDynamic(::sal_Int32 nActionID, const ::rtl::OUString & aApplicationName, const css::uno::Reference< css::frame::XController > & xController, const css::lang::Locale & aLocale) throw (css::uno::RuntimeException, css::lang::IllegalArgumentException);
    virtual ::sal_Bool SAL_CALL isShowSmartTagIndicator(::sal_Int32 nActionID, const ::rtl::OUString & aApplicationName, const css::uno::Reference< css::frame::XController > & xController, const css::lang::Locale & aLocale) throw (css::uno::RuntimeException, css::lang::IllegalArgumentException);

private:
    MOSTLSmartTagAction(MOSTLSmartTagAction &); // not defined
    void operator =(MOSTLSmartTagAction &); // not defined

    virtual ~MOSTLSmartTagAction();

    void LoadSmartTagFromFile(const rtl::OUString filename);
    void xPathError( xmlXPathContextPtr xpathCtx, xmlDocPtr doc );
    rtl::OUString getXmlValue( xmlXPathContextPtr xpathCtx, char *xpath );

    css::uno::Reference< css::uno::XComponentContext > m_xContext;
    sal_Int32 m_SmartTagCount;

    std::vector< MOSTLActionData > m_aActionData;
    rtl::OUString aComponentName;
};

MOSTLSmartTagAction::MOSTLSmartTagAction(css::uno::Reference< css::uno::XComponentContext > const & context) :
    m_xContext(context), m_SmartTagCount(0)
{
}

MOSTLSmartTagAction::~MOSTLSmartTagAction()
{
}

// com.sun.star.uno.XServiceInfo:
::rtl::OUString SAL_CALL MOSTLSmartTagAction::getImplementationName() throw (css::uno::RuntimeException)
{
    return comp_MOSTLSmartTagAction::_getImplementationName();
}

::sal_Bool SAL_CALL MOSTLSmartTagAction::supportsService(::rtl::OUString const & serviceName) throw (css::uno::RuntimeException)
{
    css::uno::Sequence< ::rtl::OUString > serviceNames = comp_MOSTLSmartTagAction::_getSupportedServiceNames();
    for (::sal_Int32 i = 0; i < serviceNames.getLength(); ++i) {
        if (serviceNames[i] == serviceName)
            return sal_True;
    }
    return sal_False;
}

css::uno::Sequence< ::rtl::OUString > SAL_CALL MOSTLSmartTagAction::getSupportedServiceNames() throw (css::uno::RuntimeException)
{
    return comp_MOSTLSmartTagAction::_getSupportedServiceNames();
}

// ::com::sun::star::lang::XInitialization:
void SAL_CALL MOSTLSmartTagAction::initialize(const css::uno::Sequence< ::com::sun::star::uno::Any > &) throw (css::uno::RuntimeException, css::uno::Exception)
{
    std::vector<rtl::OUString> aFileList;

    rtl::OUString aSharedData;
    ::utl::Bootstrap::locateSharedData( aSharedData );
    lcl_GetFileList( aSharedData, aFileList );

    rtl::OUString aUserData;
    ::utl::Bootstrap::locateUserData( aUserData );
    lcl_GetFileList( aUserData, aFileList );

    for ( sal_uInt32 i = 0; i < aFileList.size(); ++i )
        LoadSmartTagFromFile( aFileList[i] );
}

// ::com::sun::star::smarttags::XSmartTagAction:
::sal_Int32 SAL_CALL MOSTLSmartTagAction::getSmartTagCount() throw (css::uno::RuntimeException)
{
    return m_SmartTagCount;
}

::rtl::OUString SAL_CALL MOSTLSmartTagAction::getName(const css::lang::Locale & /*aLocale*/) throw (css::uno::RuntimeException)
{
    return aComponentName;
}

::rtl::OUString SAL_CALL MOSTLSmartTagAction::getDescription(const css::lang::Locale & /*aLocale*/) throw (css::uno::RuntimeException)
{
    return rtl::OUString();
}

::rtl::OUString SAL_CALL MOSTLSmartTagAction::getSmartTagName(::sal_Int32 nSmartTagIndex) throw (css::uno::RuntimeException, css::lang::IndexOutOfBoundsException)
{
    if ( nSmartTagIndex < 0 || nSmartTagIndex >= m_SmartTagCount )
        return rtl::OUString();

    return m_aActionData[nSmartTagIndex].getSmartTagType();
}

::rtl::OUString SAL_CALL MOSTLSmartTagAction::getSmartTagCaption(::sal_Int32 nSmartTagIndex,
                                                                   const css::lang::Locale & /*aLocale*/) throw (css::uno::RuntimeException, css::lang::IndexOutOfBoundsException)
{
    if ( nSmartTagIndex < 0 || nSmartTagIndex >= m_SmartTagCount )
        return rtl::OUString();

    return m_aActionData[nSmartTagIndex].getCaption();
}

::sal_Int32 SAL_CALL MOSTLSmartTagAction::getActionCount( const ::rtl::OUString& aSmartTagName,
                                                          const css::uno::Reference< css::frame::XController > & /*xController*/) throw (css::uno::RuntimeException)
{
    for ( sal_uInt32 i = 0; i < m_aActionData.size(); ++i )
    {
        if ( aSmartTagName.equals( m_aActionData[i].getSmartTagType() ) )
            return m_aActionData[i].getActionCount();
    }

    return 0;
}

::sal_Int32 SAL_CALL MOSTLSmartTagAction::getActionID( const ::rtl::OUString & aSmartTagName,
                                                       ::sal_Int32 nActionIndex,
                                                       const css::uno::Reference< css::frame::XController > & /*xController*/ ) throw (css::uno::RuntimeException, css::lang::IllegalArgumentException)
{
    for ( sal_uInt32 i = 0; i < m_aActionData.size(); ++i )
    {
        if ( aSmartTagName.equals( m_aActionData[i].getSmartTagType() ) )
             return m_aActionData[i].getActionID( nActionIndex );
    }

    return 0;
}

::rtl::OUString SAL_CALL MOSTLSmartTagAction::getActionCaptionFromID( ::sal_Int32 nActionID, const ::rtl::OUString & /*aApplicationName*/,
                                                                      const css::lang::Locale & /*aLocale*/,
                                                                      const css::uno::Reference< css::container::XStringKeyMap > & /*xProperties*/,
                                                                      const ::rtl::OUString & /*aText*/,
                                                                      const ::rtl::OUString & /*aXml*/,
                                                                      const css::uno::Reference< css::frame::XController > & /*xController*/,
                                                                      const css::uno::Reference< css::text::XTextRange > & /*xTarget*/) throw (css::uno::RuntimeException, css::lang::IllegalArgumentException)
{
    for ( sal_uInt32 i = 0; i < m_aActionData.size(); ++i )
    {
        const sal_Int32 nActionCount = m_aActionData[i].getActionCount();
        for ( sal_Int32 j = 0; j < nActionCount; ++j )
        {
            if ( m_aActionData[i].getActionID( j ) == nActionID )
                return m_aActionData[i].getActionCaption( j );
        }
    }

    return rtl::OUString();
}

::rtl::OUString SAL_CALL MOSTLSmartTagAction::getActionNameFromID(::sal_Int32 /*ActionID*/, const css::uno::Reference< css::frame::XController > & /*Controller*/) throw (css::uno::RuntimeException, css::lang::IllegalArgumentException)
{
    return C2U( "This should be a language independent caption" );
}

void SAL_CALL MOSTLSmartTagAction::invokeAction(::sal_Int32 nActionID,
                                                  const ::rtl::OUString & /*aApplicationName*/,
                                                  const css::uno::Reference< css::frame::XController > & /*xController*/,
                                                  const css::uno::Reference< css::text::XTextRange > & xTarget,
                                                  const css::uno::Reference< css::container::XStringKeyMap > & /*xProperties*/,
                                                  const ::rtl::OUString & /*aText*/,
                                                  const ::rtl::OUString & /*aXml*/,
                                                  const css::lang::Locale & /*aLocale*/) throw (css::uno::RuntimeException, css::lang::IllegalArgumentException)
{
    if (!m_aActionData.size() ) return;

    rtl::OUString aWord = xTarget->getString();

    // open URL in browser
    css::uno::Reference< css::system::XSystemShellExecute > xSystemShellExecute( m_xContext->getServiceManager()->createInstanceWithContext( C2U( "com.sun.star.system.SystemShellExecute" ), m_xContext ), css::uno::UNO_QUERY );

    if ( xSystemShellExecute.is() )
    {
        rtl::OUString aURLString;

        // get action url:
        for ( sal_uInt32 i = 0; i < m_aActionData.size(); ++i )
        {
            const sal_uInt32 nActionCount = m_aActionData[i].getActionCount();
            for ( sal_uInt32 j = 0; j < nActionCount; ++j )
            {
                if ( m_aActionData[i].getActionID( j ) == nActionID )
                    aURLString = m_aActionData[i].getActionUrl( j );
            }
        }

        // replace {TEXT} in URL
        sal_Int32 nIdx = aURLString.indexOf(C2U("{TEXT}"));
        if (nIdx != -1)
            aURLString = aURLString.replaceAt(nIdx, 6, aWord);

        try
        {
            // start browser
            xSystemShellExecute->execute( aURLString, rtl::OUString(), css::system::SystemShellExecuteFlags::DEFAULTS );
        }
        catch( css::uno::Exception& )
        {
        }
    }
}

::sal_Bool SAL_CALL MOSTLSmartTagAction::isCaptionDynamic(::sal_Int32 /*ActionID*/,
                                                            const ::rtl::OUString & /*ApplicationName*/,
                                                            const css::uno::Reference< css::frame::XController > & /*Controller*/,
                                                            const css::lang::Locale & /*Locale*/) throw (css::uno::RuntimeException, css::lang::IllegalArgumentException)
{
    return sal_False;
}

::sal_Bool SAL_CALL MOSTLSmartTagAction::isShowSmartTagIndicator(::sal_Int32 /*ActionID*/,
                                                                   const ::rtl::OUString & /*ApplicationName*/,
                                                                   const css::uno::Reference< css::frame::XController > & /*Controller*/,
                                                                   const css::lang::Locale & /*Locale*/) throw (css::uno::RuntimeException, css::lang::IllegalArgumentException)
{
    return sal_False;
}

} // closing anonymous implementation namespace



// component helper namespace
namespace comp_MOSTLSmartTagAction {

::rtl::OUString SAL_CALL _getImplementationName() {
    return ::rtl::OUString(RTL_CONSTASCII_USTRINGPARAM(
        "MOSTLSmartTagAction"));
}

css::uno::Sequence< ::rtl::OUString > SAL_CALL _getSupportedServiceNames()
{
    css::uno::Sequence< ::rtl::OUString > s(1);
    s[0] = ::rtl::OUString(RTL_CONSTASCII_USTRINGPARAM(
        "com.sun.star.smarttags.SmartTagAction"));
    return s;
}

css::uno::Reference< css::uno::XInterface > SAL_CALL _create(
    const css::uno::Reference< css::uno::XComponentContext > & context)
        SAL_THROW((css::uno::Exception))
{
    return static_cast< ::cppu::OWeakObject * >(new MOSTLSmartTagAction(context));
}

} // closing component helper namespace

static ::cppu::ImplementationEntry const entries[] = {
    { &comp_MOSTLSmartTagAction::_create,
      &comp_MOSTLSmartTagAction::_getImplementationName,
      &comp_MOSTLSmartTagAction::_getSupportedServiceNames,
      &::cppu::createSingleComponentFactory, 0, 0 },
    { 0, 0, 0, 0, 0, 0 }
};

extern "C" void SAL_CALL component_getImplementationEnvironment(
    const char ** envTypeName, uno_Environment **)
{
    *envTypeName = CPPU_CURRENT_LANGUAGE_BINDING_NAME;
}

extern "C" void * SAL_CALL component_getFactory(
    const char * implName, void * serviceManager, void * registryKey)
{
    return ::cppu::component_getFactoryHelper(
        implName, serviceManager, registryKey, entries);
}

extern "C" sal_Bool SAL_CALL component_writeInfo(
    void * serviceManager, void * registryKey)
{
    return ::cppu::component_writeInfoHelper(serviceManager, registryKey, entries);
}

void MOSTLSmartTagAction::LoadSmartTagFromFile(const rtl::OUString newfilename)
{
    const rtl::OString ostring( ::rtl::OUStringToOString (newfilename, RTL_TEXTENCODING_UTF8) );

    xmlDocPtr doc = xmlParseFile(ostring.getStr());

    if (doc == NULL )
    {
        fprintf(stderr,"Document not parsed successfully. \n");
        return;
    }

    /* Create xpath evaluation context */
    xmlXPathContextPtr xpathCtx = xmlXPathNewContext(doc);
    if(xpathCtx == NULL)
    {
      fprintf(stderr,"Error: unable to create new XPath context\n");
      xmlFreeDoc(doc);
      return;
    }

    /* register namespace */
    if(xmlXPathRegisterNs(xpathCtx, (xmlChar*) "FL", (xmlChar*) "http://schemas.microsoft.com/office/smarttags/2003/mostl") != 0)
    {
        fprintf(stderr,"Error: unable to register NS with prefix=\"%s\" and href=\"%s\"\n", "", "");
        xmlFreeDoc(doc);
        return;
    }

    /* get name */
    aComponentName = C2U("Smart Tag List Action");/*getXmlValue(xpathCtx, "/FL:smarttaglist/FL:name");*/

    /* get smarttag count */
    xmlXPathObjectPtr xpathObj = xmlXPathEvalExpression((xmlChar*) "/FL:smarttaglist/FL:smarttag", xpathCtx);
    if(xpathObj == NULL)
    {
        xPathError(xpathCtx, doc);
        return;
    }

    xmlNodeSetPtr nodes = xpathObj->nodesetval;
    int nNumberOfSmartTagTypes = (nodes) ? nodes->nodeNr : 0;

    m_SmartTagCount += nNumberOfSmartTagTypes;

    xmlXPathFreeObject(xpathObj);

    if ( !nNumberOfSmartTagTypes )
        return;

    // iterate over all smarttags in found in xml file
    for( int j = 1; j <= nNumberOfSmartTagTypes; ++j )
    {
        MOSTLActionData aData;

        /* set caption */
        sal_Char xpath[64];
        snprintf(xpath, sizeof(xpath), "/FL:smarttaglist/FL:smarttag[%d]/FL:caption", j);
        aData.setCaption(getXmlValue(xpathCtx, xpath));

        /* get action library name */
        snprintf(xpath, sizeof(xpath), "/FL:smarttaglist/FL:smarttag[%d]/@type", j);
        aData.setSmartTagType(getXmlValue(xpathCtx, xpath));

        /* get actions */
        snprintf(xpath, sizeof(xpath), "/FL:smarttaglist/FL:smarttag[%d]/FL:actions/FL:action", j);
        xpathObj = xmlXPathEvalExpression((xmlChar*) xpath, xpathCtx);
        if(xpathObj == NULL)
        {
            xPathError(xpathCtx, doc);
            return;
        }

        nodes = xpathObj->nodesetval;
        int size = (nodes) ? nodes->nodeNr : 0;

        for( int i = 0; i < size; ++i )
        {
            const sal_Char* pCap = NULL;
            const sal_Char* pUrl = NULL;
            for ( xmlNode* cur_node = nodes->nodeTab[i]->children; cur_node; cur_node = cur_node->next )
            {
                if (cur_node->type == XML_ELEMENT_NODE)
                {
                    if (!strcmp((const char*) cur_node->name, "caption"))
                        pCap = (const sal_Char*) cur_node->children->content;
                    if (!strcmp((const char*) cur_node->name, "url"))
                        pUrl = (const sal_Char*) cur_node->children->content;
                }
            }

            rtl::OUString aCap = rtl::OUString::createFromAscii("");
            if (pCap)
                aCap = rtl::OUString(pCap, strlen(pCap), RTL_TEXTENCODING_UTF8);

            rtl::OUString aUrl = C2U("");
            if (pUrl)
                aUrl = rtl::OUString(pUrl, strlen(pUrl), RTL_TEXTENCODING_UTF8);

            static sal_Int32 nActionID = 500;
            MOSTLAction ma(aCap, aUrl, nActionID++);
            aData.addAction(ma);
        }

        xmlXPathFreeObject(xpathObj);

        m_aActionData.push_back( aData );
    }

    xmlXPathFreeContext(xpathCtx);
    xmlFreeDoc(doc);
}

void MOSTLSmartTagAction::xPathError(xmlXPathContextPtr xpathCtx, xmlDocPtr doc)
{
     fprintf(stderr,"Error: unable to evaluate xpath expression.\n");
     xmlXPathFreeContext(xpathCtx);
     xmlFreeDoc(doc);
}

rtl::OUString MOSTLSmartTagAction::getXmlValue(xmlXPathContextPtr xpathCtx, char *xpath)
{
    xmlXPathObjectPtr xpathObj = xmlXPathEvalExpression((xmlChar*) xpath, xpathCtx);
    if(xpathObj == NULL)
        return C2U("");

    xmlNodeSetPtr nodes = xpathObj->nodesetval;
    int size = (nodes) ? nodes->nodeNr : 0;

    rtl::OUString aValue = C2U("");
    if (size>0)
    {
        const sal_Char* pTmp= (const sal_Char*) nodes->nodeTab[0]->children->content;
        aValue = C2U(pTmp);
    }

    xmlXPathFreeObject(xpathObj);

    return aValue;
}

rtl::OUString MOSTLActionData::getActionCaption(sal_Int32 nActionIndex) const
{
    if ((nActionIndex < 0) || ((sal_uInt32)nActionIndex >= aActions.size()))
        return C2U("");

    return aActions[nActionIndex].getCaption();
}

rtl::OUString MOSTLActionData::getActionUrl(sal_Int32 nActionIndex) const
{
    if ((nActionIndex < 0) || ((sal_uInt32)nActionIndex >= aActions.size()))
        return C2U("");

    return aActions[nActionIndex].getUrl();
}

sal_Int32 MOSTLActionData::getActionID(sal_Int32 nActionIndex) const
{
    if ((nActionIndex < 0) || ((sal_uInt32)nActionIndex >= aActions.size()))
        return 0;

    return aActions[nActionIndex].getActionID();
}

